from __future__ import annotations

from digitalhub_core.entities._base.spec.base import Spec, SpecParams


class WorkflowSpec(Spec):
    """
    Workflow specifications.
    """


class WorkflowParams(SpecParams):
    """
    Workflow parameters.
    """
